/****************************************************************************

	DirMgr.h

	This file contains the C declarations and definitions for the
	DirectoryMgr system.

	This system is intended to managed filtered and sorted directory
	lists.

 ****************************************************************************/

/*  Changed Newmarch, Dec 92 - added func prototypes to some functions */

#ifndef _DIRECTORY_MGR_H_
#define	_DIRECTORY_MGR_H_

#include "Directory.h"

#ifndef NO_REGCOMP
#include <regex.h>
#endif

/*---------------------------------------------------------------------------*

        Simple DirectoryMgr Interface

		DirectoryMgrSimpleOpen();
		DirectoryMgrSimpleRefilter();
		DirectoryMgrSimpleResort();

	Standard DirectoryMgr Interface

		DirectoryMgrOpen();
		DirectoryMgrClose();
		DirectoryMgrRefilter();
		DirectoryMgrRefresh();
		DirectoryMgrResort();

	Moving Around Items

		DirectoryMgrGotoItem();
		DirectoryMgrGotoNamedItem();
		DirectoryMgrRestart();
		DirectoryMgrGetIndex();
		DirectoryMgrCurrentEntry();
		DirectoryMgrNextEntry();
		DirectoryMgrPrevEntry();

	Utility Functions

		DirectoryMgrSimpleFilterFunc();
		DirectoryMgrSimpleSortingFunc();

	Comparison Functions

		DirectoryMgrCompareName();
		DirectoryMgrCompareSizeAscending();
		DirectoryMgrCompareSizeDescending();

	Macros

		DirectoryMgrDir();
		DirectoryMgrData();
		DirectoryMgrSortedPtrs();
		DirectoryMgrFilterFunc();
		DirectoryMgrCompFunc();
		DirectoryMgrFilterData();
		DirectoryMgrFreeFilterData();
		DirectoryMgrTotalCount();
		DirectoryMgrFilteredCount();
		DirectoryMgrCurrentIndex();


 *---------------------------------------------------------------------------*/

#ifndef PFI
#ifdef UseFunctionProtoypes
typedef int (*PFI) ();
#else
typedef int (*PFI)();
#endif
#endif

#ifdef UseFunctionPrototypes
typedef int (*PSortFunc) (DirEntry **, DirEntry **);
#else
typedef int (*PSortFunc) ();
#endif

#ifdef UseFunctionPrototypes
#   ifndef NO_REGCOMP
typedef int (*PFilterFunc) (DirEntry *, regex_t *);
#   else
typedef int (*PFilterFunc) (DirEntry *, char *);
#   endif
#else
typedef int (*PFilterFunc) ();
#endif



typedef struct entry_cons
{
	DIR_ENTRY dir_entry;
	struct entry_cons *next;
} DIR_ENTRY_CONS;

typedef	DIR_ENTRY_CONS DirEntryCons;

typedef struct
{
	DIRECTORY dir;
	DIR_ENTRY *data;
	DIR_ENTRY **sorted_ptrs;
	int total_count;
	int filtered_count;
	PFilterFunc filter_func;
#ifndef NO_REGCOMP
	regex_t *filter_data;
#else
	char *filter_data;
#endif
	int free_filter_data;
	PSortFunc comp_func;
	int current_index;
} DIRECTORY_MGR;

typedef	DIRECTORY_MGR DirectoryMgr;

#define	DIR_MGR_SORT_NONE			0
#define	DIR_MGR_SORT_NAME			1
#define	DIR_MGR_SORT_NAME_DIRS_FIRST		2
#define DIR_MGR_SORT_SIZE_ASCENDING		3
#define DIR_MGR_SORT_SIZE_DESCENDING		4
#define	DIR_MGR_SORT_ACCESS_ASCENDING		5
#define	DIR_MGR_SORT_ACCESS_DESCENDING		6

#define	DirectoryMgrDir(dm)		(&((dm)->dir))
#define	DirectoryMgrData(dm)		((dm)->data)
#define	DirectoryMgrSortedPtrs(dm)	((dm)->sorted_ptrs)
#define	DirectoryMgrFilterFunc(dm)	((dm)->filter_func)
#define	DirectoryMgrCompFunc(dm)	((dm)->comp_func)
#define	DirectoryMgrFilterData(dm)	((dm)->filter_data)
#define	DirectoryMgrFreeFilterData(dm)	((dm)->free_filter_data)
#define	DirectoryMgrTotalCount(dm)	((dm)->total_count)
#define	DirectoryMgrFilteredCount(dm)	((dm)->filtered_count)
#define	DirectoryMgrCurrentIndex(dm)	((dm)->current_index)

/* this used to be #ifndef __cplusplus - J Newmarch */
#ifndef UseFunctionPrototypes

DirectoryMgr		*DirectoryMgrSimpleOpen();
int			DirectoryMgrSimpleRefilter();
int			DirectoryMgrSimpleResort();

DirectoryMgr		*DirectoryMgrOpen();
void			DirectoryMgrClose();
void			DirectoryMgrRefilter();
int			DirectoryMgrRefresh();
void			DirectoryMgrResort();

int			DirectoryMgrGotoItem();
int			DirectoryMgrGotoNamedItem();
void			DirectoryMgrRestart();
int			DirectoryMgrGetIndex();
DirEntry		*DirectoryMgrCurrentEntry();
DirEntry		*DirectoryMgrNextEntry();
DirEntry		*DirectoryMgrPrevEntry();

int			DirectoryMgrSimpleFilterFunc();
int			DirectoryMgrSimpleSortingFunc();

int			DirectoryMgrCompareName();
int			DirectoryMgrCompareNameDirsFirst();
int			DirectoryMgrCompareSizeAscending();
int			DirectoryMgrCompareSizeDescending();
int			DirectoryMgrCompareLastAccessAscending();
int			DirectoryMgrCompareLastAccessDescending();

int			DirectoryMgrFilterName();

#else /* UseFunctionPrototypes */
#ifndef NO_REGCOMP
DirectoryMgr		*DirectoryMgrSimpleOpen(char *, int, char*);
int			DirectoryMgrSimpleRefilter(DirectoryMgr *, char *);
int			DirectoryMgrSimpleResort(DirectoryMgr *, int);

DirectoryMgr		*DirectoryMgrOpen(char *, PSortFunc, PFilterFunc, regex_t *, int);
void			DirectoryMgrClose(DirectoryMgr *);
void			DirectoryMgrRefilter(DirectoryMgr *, PFilterFunc, regex_t *, int);
int			DirectoryMgrRefresh(DirectoryMgr *);
void			DirectoryMgrResort(DirectoryMgr *, PSortFunc);

int			DirectoryMgrGotoItem(DirectoryMgr *, int);
int			DirectoryMgrGotoNamedItem(DirectoryMgr *, char *);
void			DirectoryMgrRestart(DirectoryMgr *);
int			DirectoryMgrGetIndex(DirectoryMgr *);
DirEntry		*DirectoryMgrCurrentEntry(DirectoryMgr *);
DirEntry		*DirectoryMgrNextEntry(DirectoryMgr *);
DirEntry		*DirectoryMgrPrevEntry(DirectoryMgr *);

int			DirectoryMgrSimpleSortingFunc(int, PSortFunc *);
int			DirectoryMgrSimpleFilterFunc(char *, PFilterFunc *, regex_t **);

int			DirectoryMgrCompareName(DirEntry **, DirEntry **);
int			DirectoryMgrCompareNameDirsFirst(DirEntry **, DirEntry **);
int			DirectoryMgrCompareSizeAscending(DirEntry **, DirEntry **);
int			DirectoryMgrCompareSizeDescending(DirEntry **, DirEntry **);
int			DirectoryMgrCompareLastAccessAscending(DirEntry **, DirEntry **);
int			DirectoryMgrCompareLastAccessDescending(DirEntry **, DirEntry **);

int			DirectoryMgrFilterName(DirEntry *, regex_t *);
#else /* NO_REGCOMP */
DirectoryMgr		*DirectoryMgrSimpleOpen(char *, int, char*);
int			DirectoryMgrSimpleRefilter(DirectoryMgr *, char *);
int			DirectoryMgrSimpleResort(DirectoryMgr *, int);

DirectoryMgr		*DirectoryMgrOpen(char *, PSortFunc, PFilterFunc, char *, int);
void			DirectoryMgrClose(DirectoryMgr *);
void			DirectoryMgrRefilter(DirectoryMgr *, PFilterFunc, char *, int);
int			DirectoryMgrRefresh(DirectoryMgr *);
void			DirectoryMgrResort(DirectoryMgr *, PSortFunc);

int			DirectoryMgrGotoItem(DirectoryMgr *, int);
int			DirectoryMgrGotoNamedItem(DirectoryMgr *, char *);
void			DirectoryMgrRestart(DirectoryMgr *);
int			DirectoryMgrGetIndex(DirectoryMgr *);
DirEntry		*DirectoryMgrCurrentEntry(DirectoryMgr *);
DirEntry		*DirectoryMgrNextEntry(DirectoryMgr *);
DirEntry		*DirectoryMgrPrevEntry(DirectoryMgr *);

int			DirectoryMgrSimpleSortingFunc(int, PSortFunc *);
int			DirectoryMgrSimpleFilterFunc(char *, PFilterFunc *, char **);

int			DirectoryMgrCompareName(DirEntry **, DirEntry **);
int			DirectoryMgrCompareNameDirsFirst(DirEntry **, DirEntry **);
int			DirectoryMgrCompareSizeAscending(DirEntry **, DirEntry **);
int			DirectoryMgrCompareSizeDescending(DirEntry **, DirEntry **);
int			DirectoryMgrCompareLastAccessAscending(DirEntry **, DirEntry **);
int			DirectoryMgrCompareLastAccessDescending(DirEntry **, DirEntry **);

int			DirectoryMgrFilterName(DirEntry *, char *);
#endif /* NO_REGCOMP */

#endif

#endif
